==> lowcode-plugin-code-editor-3.0.2.tgz: package/es/components/JSEditor/JsEditor.js <==
import _asyncToGenerator from "@babel/runtime/helpers/asyncToGenerator";
import _inheritsLoose from "@babel/runtime/helpers/inheritsLoose";
import _regeneratorRuntime from "@babel/runtime/regenerator";
import React, { PureComponent } from 'react';
import MonacoEditor from '@gant-lowcode/lowcode-plugin-base-monaco-editor';
import { ErrorTip } from '../ErrorTip';
import { defaultBabelConfig, defaultCode, WORDS } from '../../config';
import { transformJS, transformAst } from '../../utils';
import { statementParser } from '../../utils/statement-parser';
import { extraLibs } from '../../extraLibsMap';
import './JsEditor.css';
var LIFECYCLES_FUNCTION_MAP = {
  react: ['constructor', 'render', 'componentDidMount', 'componentDidUpdate', 'componentWillUnmount', 'componentDidCatch']
};
export var JsEditor = /*#__PURE__*/function (_PureComponent) {
  function JsEditor() {
    var _this$props$jsCode;
    var _this;
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }
    _this = _PureComponent.call.apply(_PureComponent, [this].concat(args)) || this;
    _this.state = {
      errorInfo: null,
      hasError: false,
      code: (_this$props$jsCode = _this.props.jsCode) !== null && _this$props$jsCode !== void 0 ? _this$props$jsCode : defaultCode
    };
    _this.monaco = void 0;
    _this.monacoEditor = void 0;
    _this.lastErrorDecoration = void 0;
    _this.disposeProvider = void 0;
    _this.editorDidMount = /*#__PURE__*/function () {
      var _ref = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee(monaco, editor) {
        var _this$disposeProvider, _this$disposeProvider2;
        var internalModules;
        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) switch (_context.prev = _context.next) {
            case 0:
              _this.monacoEditor = editor;
              _this.monaco = monaco;
              monaco.languages.typescript.typescriptDefaults.setDiagnosticsOptions({
                noSemanticValidation: false,
                noSyntaxValidation: false,
                noSuggestionDiagnostics: true
              });
              monaco.languages.typescript.typescriptDefaults.setCompilerOptions({
                target: monaco.languages.typescript.ScriptTarget.ESNext,
                allowNonTsExtensions: true,
                moduleResolution: monaco.languages.typescript.ModuleResolutionKind.NodeJs,
                module: monaco.languages.typescript.ModuleKind.ESNext,
                noEmit: true,
                esModuleInterop: true,
                jsx: monaco.languages.typescript.JsxEmit.React,
                reactNamespace: 'React',
                allowJs: true,
                typeRoots: ['node_modules/@types'],
                baseUrl: '.'
              });

              // 将 babel 报错提示在这里
              (_this$disposeProvider = _this.disposeProvider) === null || _this$disposeProvider === void 0 ? void 0 : (_this$disposeProvider2 = _this$disposeProvider.dispose) === null || _this$disposeProvider2 === void 0 ? void 0 : _this$disposeProvider2.call(_this$disposeProvider);
              _this.disposeProvider = monaco.languages.registerHoverProvider('typescript', {
                provideHover: function provideHover(model, position) {
                  var _this$state$errorLoca;
                  if (((_this$state$errorLoca = _this.state.errorLocation) === null || _this$state$errorLoca === void 0 ? void 0 : _this$state$errorLoca.line) === position.lineNumber) {
                    var _this$state$errorLoca2, _this$state$errorLoca3, _this$state$errorLoca4;
                    return {
                      range: new monaco.Range(position.lineNumber, (_this$state$errorLoca2 = _this.state.errorLocation) === null || _this$state$errorLoca2 === void 0 ? void 0 : _this$state$errorLoca2.column, position.lineNumber, ((_this$state$errorLoca3 = (_this$state$errorLoca4 = _this.state.errorLocation) === null || _this$state$errorLoca4 === void 0 ? void 0 : _this$state$errorLoca4.column) !== null && _this$state$errorLoca3 !== void 0 ? _this$state$errorLoca3 : 0) + 1),
                      contents: [{
                        value: "**" + _this.state.errorInfo + "**",
                        supportHtml: true,
                        isTrusted: true
                      }]
                    };
                  }
                  return null;
                }
              });
              monaco.languages.typescript.typescriptDefaults.addExtraLib("\n      declare module 'ahooks' {\n        export function useRequest<TData, TParams extends any[]>(\n          service: (...args: TParams) => Promise<TData>,\n          options?: {\n            manual?: boolean;\n            defaultParams?: TParams;\n          }\n        ): {\n          loading: boolean;\n          data?: TData;\n          error?: Error;\n          run: (...args: TParams) => Promise<TData>;\n        };\n      }\n    ", 'file:///node_modules/@types/ahooks/index.d.ts');
              internalModules = [{
                path: 'lowcode-common/constants.ts',
                content: "\n          const constants: Record<string, any>;\n          export default constants;\n        "
              }, {
                path: 'lowcode-common/utils.ts',
                content: "\n          const utils: Record<string, any>;\n          export default utils;\n        "
              }, {
                path: 'lowcode-common/i18n.ts',
                content: "\n          const i18n: (key: string, params?: Record<string, any>) => string;\n          export default i18n;\n        "
              }];
              internalModules.forEach(function (_ref2) {
                var path = _ref2.path,
                  content = _ref2.content;
                monaco.languages.typescript.typescriptDefaults.addExtraLib(content, "file:///src/" + path // ⚠️ 注意这要匹配 import 的路径
                );
              });
              extraLibs.forEach(function (_ref3) {
                var filePath = _ref3[0],
                  content = _ref3[1];
                monaco.languages.typescript.typescriptDefaults.addExtraLib(content, filePath);
              });
              console.log('getCompilerOptions', monaco.languages.typescript.typescriptDefaults.getCompilerOptions());
            case 11:
            case "end":
              return _context.stop();
          }
        }, _callee);
      }));
      return function (_x, _x2) {
        return _ref.apply(this, arguments);
      };
    }();
    _this._updateCode = function (newCode) {
      console.log('--defaultBabelConfig', defaultBabelConfig);
      var _transformJS = transformJS(newCode, defaultBabelConfig),
        hasError = _transformJS.hasError,
        errorInfo = _transformJS.errorInfo,
        errorLocation = _transformJS.errorLocation;
      var _this2 = _this,
        monacoEditor = _this2.monacoEditor,
        monaco = _this2.monaco;
      if (!monacoEditor || !monaco) {
        return;
      }
      _this.props.setChanged(newCode !== _this.props.jsCode);
      // const pos = monacoEditor.getPosition();
      _this.setState({
        errorInfo: errorInfo,
        hasError: hasError,
        code: newCode,
        errorLocation: errorLocation
      }, function () {
        // monacoEditor.setPosition(pos);
        // update error decorations
        if (_this.lastErrorDecoration) {
          monacoEditor.deltaDecorations(_this.lastErrorDecoration, []);
          _this.lastErrorDecoration = null;
        }
        if (hasError && errorLocation) {
          var _this$monaco, _this$monaco$editor;
          _this.lastErrorDecoration = monacoEditor.deltaDecorations([], [{
            range: new monaco.Range(errorLocation.line, errorLocation.column, errorLocation.line, errorLocation.column),
            options: {
              inlineClassName: 'squiggly-error'
            },
            minimap: {
              color: {
                id: 'minimap.errorHighlight'
              },
              position: (_this$monaco = _this.monaco) === null || _this$monaco === void 0 ? void 0 : (_this$monaco$editor = _this$monaco.editor) === null || _this$monaco$editor === void 0 ? void 0 : _this$monaco$editor.MinimapPosition.Inline
            }
          }, {
            range: new monaco.Range(errorLocation.line, errorLocation.column, errorLocation.line, errorLocation.column),
            options: {
              isWholeLine: true,
              linesDecorationsClassName: 'plugin-code-editor-error-line'
            }
          }]);
        }
      });
    };
    return _this;
  }
  _inheritsLoose(JsEditor, _PureComponent);
  var _proto = JsEditor.prototype;
  // get schema from code
  _proto.getSchemaFromCode = function getSchemaFromCode() {
    var _this$monacoEditor$ge, _this$monacoEditor, _this$monacoEditor$ge2;
    var code = (_this$monacoEditor$ge = (_this$monacoEditor = this.monacoEditor) === null || _this$monacoEditor === void 0 ? void 0 : (_this$monacoEditor$ge2 = _this$monacoEditor.getModel()) === null || _this$monacoEditor$ge2 === void 0 ? void 0 : _this$monacoEditor$ge2.getValue()) !== null && _this$monacoEditor$ge !== void 0 ? _this$monacoEditor$ge : this.state.code;
    var ast = transformAst(code);
    // const { methods, errorsByMethods } = getMethods(ast);
    console.log('--code', code);
    var _statementParser = statementParser(code, ast),
      statements = _statementParser.statements,
      errorsByStatements = _statementParser.errorsByStatements;
    // let lifeCycles: Record<string, unknown> = {};
    // let newMethods: Record<string, unknown> = {};
    console.log('--statements', statements);
    // Object.keys(methods).forEach((method) => {
    //   if (LIFECYCLES_FUNCTION_MAP.react.indexOf(method) >= 0) {
    //     lifeCycles[method] = methods[method];
    //   } else {
    //     newMethods[method] = methods[method];
    //   }
    // });

    if (Object.keys(errorsByStatements).length > 0) {
      var config = {
        title: WORDS.title,
        content: /*#__PURE__*/React.createElement(React.Fragment, null, WORDS.functionParseError, Object.entries(errorsByStatements).map(function (_ref4, index) {
          var key = _ref4[0],
            err = _ref4[1];
          return /*#__PURE__*/React.createElement("p", {
            key: index
          }, key, ": ", err);
        }))
      };
      this.props.modal.error(config);
    }
    return {
      // state: stateParser(ast),
      // methods: newMethods,
      // lifeCycles,
      statements: statements,
      originCode: code
    };
  };
  _proto.focusByFunctionName = function focusByFunctionName(_ref5) {
    var _monacoEditor$getMode, _monacoEditor$getMode2;
    var functionName = _ref5.functionName;
    var monacoEditor = this.monacoEditor;
    var matchedResult = monacoEditor === null || monacoEditor === void 0 ? void 0 : (_monacoEditor$getMode = monacoEditor.getModel()) === null || _monacoEditor$getMode === void 0 ? void 0 : (_monacoEditor$getMode2 = _monacoEditor$getMode.findMatches("^\\s*(?:async)?\\s*" + functionName + "\\s*\\([\\s\\S]*\\)[\\s\\S]*\\{", false, true)) === null || _monacoEditor$getMode2 === void 0 ? void 0 : _monacoEditor$getMode2[0];
    if (matchedResult) {
      setTimeout(function () {
        monacoEditor.revealLineInCenter(matchedResult.range.startLineNumber);
        monacoEditor.setPosition({
          column: matchedResult.range.endColumn,
          lineNumber: matchedResult.range.endLineNumber
        });
        monacoEditor.focus();
      }, 100);
    }
  };
  _proto.addFunction = function addFunction(params) {
    var _monacoEditor$getMode3;
    var monacoEditor = this.monacoEditor,
      monaco = this.monaco;
    if (!monacoEditor || !monaco) {
      return;
    }

    // 找到最后一个 }，在他前面插入新的 function 字符串
    var matches = (_monacoEditor$getMode3 = monacoEditor.getModel()) === null || _monacoEditor$getMode3 === void 0 ? void 0 : _monacoEditor$getMode3.findMatches('}');
    var range = {};
    if (matches && matches.length > 0) {
      var _matches;
      var _ref6 = ((_matches = matches[matches.length - 1]) === null || _matches === void 0 ? void 0 : _matches.range) || {},
        startLineNumber = _ref6.startLineNumber,
        startColumn = _ref6.startColumn;
      range = new monaco.Range(startLineNumber, startColumn, startLineNumber, startColumn);
    }
    var functionCode = params.template ? params.template : "\n\t" + params.functionName + "(){\n\t}\n";
    monacoEditor.executeEdits('log-source', [{
      identifier: 'event_id',
      range: range,
      text: functionCode,
      forceMoveMarkers: true
    }]);
    params.functionName && this.focusByFunctionName(params);
  };
  _proto.render = function render() {
    var _this$state = this.state,
      code = _this$state.code,
      hasError = _this$state.hasError,
      errorInfo = _this$state.errorInfo;
    return /*#__PURE__*/React.createElement("div", {
      className: "plugin-code-editor-js plugin-code-editor-inner"
    }, /*#__PURE__*/React.createElement(MonacoEditor, {
      value: code,
      language: "typescript",
      height: "100%",
      path: "file:///src/lowcode/index.tsx",
      supportFullScreen: true,
      onChange: this._updateCode,
      editorDidMount: this.editorDidMount
    }), hasError && typeof errorInfo === 'string' ? /*#__PURE__*/React.createElement(ErrorTip, {
      errorInfo: errorInfo
    }) : null);
  };
  return JsEditor;
}(PureComponent);

==> lowcode-plugin-code-editor-3.0.5.tgz: package/es/components/JSEditor/JsEditor.js <==
import _asyncToGenerator from "@babel/runtime/helpers/asyncToGenerator";
import _inheritsLoose from "@babel/runtime/helpers/inheritsLoose";
import _regeneratorRuntime from "@babel/runtime/regenerator";
import React, { PureComponent } from 'react';
import MonacoEditor from '@gant-lowcode/lowcode-plugin-base-monaco-editor';
import { ErrorTip } from '../ErrorTip';
import { defaultBabelConfig, defaultCode, WORDS } from '../../config';
import { transformJS, transformAst } from '../../utils';
import { statementParser } from '../../utils/statement-parser';
import { extraLibs } from '../../extraLibsMap';
import './JsEditor.css';
var LIFECYCLES_FUNCTION_MAP = {
  react: ['constructor', 'render', 'componentDidMount', 'componentDidUpdate', 'componentWillUnmount', 'componentDidCatch']
};
export var JsEditor = /*#__PURE__*/function (_PureComponent) {
  function JsEditor() {
    var _this$props$jsCode;
    var _this;
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }
    _this = _PureComponent.call.apply(_PureComponent, [this].concat(args)) || this;
    _this.state = {
      errorInfo: null,
      hasError: false,
      code: (_this$props$jsCode = _this.props.jsCode) !== null && _this$props$jsCode !== void 0 ? _this$props$jsCode : defaultCode
    };
    _this.monaco = void 0;
    _this.monacoEditor = void 0;
    _this.lastErrorDecoration = void 0;
    _this.disposeProvider = void 0;
    _this.editorDidMount = /*#__PURE__*/function () {
      var _ref = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee(monaco, editor) {
        var _this$disposeProvider, _this$disposeProvider2;
        var internalModules;
        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) switch (_context.prev = _context.next) {
            case 0:
              _this.monacoEditor = editor;
              _this.monaco = monaco;
              monaco.languages.typescript.typescriptDefaults.setDiagnosticsOptions({
                noSemanticValidation: false,
                noSyntaxValidation: false,
                noSuggestionDiagnostics: true
              });
              monaco.languages.typescript.typescriptDefaults.setCompilerOptions({
                target: monaco.languages.typescript.ScriptTarget.ESNext,
                allowNonTsExtensions: true,
                moduleResolution: monaco.languages.typescript.ModuleResolutionKind.NodeJs,
                module: monaco.languages.typescript.ModuleKind.ESNext,
                noEmit: true,
                esModuleInterop: true,
                jsx: monaco.languages.typescript.JsxEmit.React,
                reactNamespace: 'React',
                allowJs: true,
                typeRoots: ['node_modules/@types'],
                baseUrl: '.'
              });

              // 将 babel 报错提示在这里
              (_this$disposeProvider = _this.disposeProvider) === null || _this$disposeProvider === void 0 ? void 0 : (_this$disposeProvider2 = _this$disposeProvider.dispose) === null || _this$disposeProvider2 === void 0 ? void 0 : _this$disposeProvider2.call(_this$disposeProvider);
              _this.disposeProvider = monaco.languages.registerHoverProvider('typescript', {
                provideHover: function provideHover(model, position) {
                  var _this$state$errorLoca;
                  if (((_this$state$errorLoca = _this.state.errorLocation) === null || _this$state$errorLoca === void 0 ? void 0 : _this$state$errorLoca.line) === position.lineNumber) {
                    var _this$state$errorLoca2, _this$state$errorLoca3, _this$state$errorLoca4;
                    return {
                      range: new monaco.Range(position.lineNumber, (_this$state$errorLoca2 = _this.state.errorLocation) === null || _this$state$errorLoca2 === void 0 ? void 0 : _this$state$errorLoca2.column, position.lineNumber, ((_this$state$errorLoca3 = (_this$state$errorLoca4 = _this.state.errorLocation) === null || _this$state$errorLoca4 === void 0 ? void 0 : _this$state$errorLoca4.column) !== null && _this$state$errorLoca3 !== void 0 ? _this$state$errorLoca3 : 0) + 1),
                      contents: [{
                        value: "**" + _this.state.errorInfo + "**",
                        supportHtml: true,
                        isTrusted: true
                      }]
                    };
                  }
                  return null;
                }
              });
              monaco.languages.typescript.typescriptDefaults.addExtraLib("\n      declare module 'ahooks' {\n        export function useRequest<TData, TParams extends any[]>(\n          service: (...args: TParams) => Promise<TData>,\n          options?: {\n            manual?: boolean;\n            defaultParams?: TParams;\n          }\n        ): {\n          loading: boolean;\n          data?: TData;\n          error?: Error;\n          run: (...args: TParams) => Promise<TData>;\n        };\n      }\n    ", 'file:///node_modules/@types/ahooks/index.d.ts');
              internalModules = [{
                path: 'lowcode-common/constants.ts',
                content: "\n          const constants: Record<string, any>;\n          export default constants;\n        "
              }, {
                path: 'lowcode-common/utils.ts',
                content: "\n          const utils: Record<string, any>;\n          export default utils;\n        "
              }, {
                path: 'lowcode-common/i18n.ts',
                content: "\n          const i18n: (key: string, params?: Record<string, any>) => string;\n          export default i18n;\n        "
              }];
              internalModules.forEach(function (_ref2) {
                var path = _ref2.path,
                  content = _ref2.content;
                monaco.languages.typescript.typescriptDefaults.addExtraLib(content, "file:///src/" + path // ⚠️ 注意这要匹配 import 的路径
                );
              });
              extraLibs.forEach(function (_ref3) {
                var filePath = _ref3[0],
                  content = _ref3[1];
                monaco.languages.typescript.typescriptDefaults.addExtraLib(content, filePath);
              });
              console.log('getCompilerOptions', monaco.languages.typescript.typescriptDefaults.getCompilerOptions());
            case 11:
            case "end":
              return _context.stop();
          }
        }, _callee);
      }));
      return function (_x, _x2) {
        return _ref.apply(this, arguments);
      };
    }();
    _this._updateCode = function (newCode) {
      console.log('--defaultBabelConfig', defaultBabelConfig);
      var _transformJS = transformJS(newCode, defaultBabelConfig),
        hasError = _transformJS.hasError,
        errorInfo = _transformJS.errorInfo,
        errorLocation = _transformJS.errorLocation;
      var _this2 = _this,
        monacoEditor = _this2.monacoEditor,
        monaco = _this2.monaco;
      if (!monacoEditor || !monaco) {
        return;
      }
      _this.props.setChanged(newCode !== _this.props.jsCode);
      // const pos = monacoEditor.getPosition();
      _this.setState({
        errorInfo: errorInfo,
        hasError: hasError,
        code: newCode,
        errorLocation: errorLocation
      }, function () {
        // monacoEditor.setPosition(pos);
        // update error decorations
        if (_this.lastErrorDecoration) {
          monacoEditor.deltaDecorations(_this.lastErrorDecoration, []);
          _this.lastErrorDecoration = null;
        }
        if (hasError && errorLocation) {
          var _this$monaco, _this$monaco$editor;
          _this.lastErrorDecoration = monacoEditor.deltaDecorations([], [{
            range: new monaco.Range(errorLocation.line, errorLocation.column, errorLocation.line, errorLocation.column),
            options: {
              inlineClassName: 'squiggly-error'
            },
            minimap: {
              color: {
                id: 'minimap.errorHighlight'
              },
              position: (_this$monaco = _this.monaco) === null || _this$monaco === void 0 ? void 0 : (_this$monaco$editor = _this$monaco.editor) === null || _this$monaco$editor === void 0 ? void 0 : _this$monaco$editor.MinimapPosition.Inline
            }
          }, {
            range: new monaco.Range(errorLocation.line, errorLocation.column, errorLocation.line, errorLocation.column),
            options: {
              isWholeLine: true,
              linesDecorationsClassName: 'plugin-code-editor-error-line'
            }
          }]);
        }
      });
    };
    return _this;
  }
  _inheritsLoose(JsEditor, _PureComponent);
  var _proto = JsEditor.prototype;
  // get schema from code
  _proto.getSchemaFromCode = function getSchemaFromCode() {
    var _this$monacoEditor$ge, _this$monacoEditor, _this$monacoEditor$ge2;
    var code = (_this$monacoEditor$ge = (_this$monacoEditor = this.monacoEditor) === null || _this$monacoEditor === void 0 ? void 0 : (_this$monacoEditor$ge2 = _this$monacoEditor.getModel()) === null || _this$monacoEditor$ge2 === void 0 ? void 0 : _this$monacoEditor$ge2.getValue()) !== null && _this$monacoEditor$ge !== void 0 ? _this$monacoEditor$ge : this.state.code;
    var ast = transformAst(code);
    // const { methods, errorsByMethods } = getMethods(ast);
    // console.log('--code', code);
    var _statementParser = statementParser(code, ast),
      statements = _statementParser.statements,
      errorsByStatements = _statementParser.errorsByStatements;
    // let lifeCycles: Record<string, unknown> = {};
    // let newMethods: Record<string, unknown> = {};
    // console.log('--statements', statements);
    // Object.keys(methods).forEach((method) => {
    //   if (LIFECYCLES_FUNCTION_MAP.react.indexOf(method) >= 0) {
    //     lifeCycles[method] = methods[method];
    //   } else {
    //     newMethods[method] = methods[method];
    //   }
    // });

    if (Object.keys(errorsByStatements).length > 0) {
      var config = {
        title: WORDS.title,
        content: /*#__PURE__*/React.createElement(React.Fragment, null, WORDS.functionParseError, Object.entries(errorsByStatements).map(function (_ref4, index) {
          var key = _ref4[0],
            err = _ref4[1];
          return /*#__PURE__*/React.createElement("p", {
            key: index
          }, key, ": ", err);
        }))
      };
      this.props.modal.error(config);
    }
    return {
      // state: stateParser(ast),
      // methods: newMethods,
      // lifeCycles,
      statements: statements,
      originCode: code
    };
  };
  _proto.focusByFunctionName = function focusByFunctionName(_ref5) {
    var _monacoEditor$getMode, _monacoEditor$getMode2;
    var functionName = _ref5.functionName;
    var monacoEditor = this.monacoEditor;
    var matchedResult = monacoEditor === null || monacoEditor === void 0 ? void 0 : (_monacoEditor$getMode = monacoEditor.getModel()) === null || _monacoEditor$getMode === void 0 ? void 0 : (_monacoEditor$getMode2 = _monacoEditor$getMode.findMatches("^\\s*(?:async)?\\s*" + functionName + "\\s*\\([\\s\\S]*\\)[\\s\\S]*\\{", false, true)) === null || _monacoEditor$getMode2 === void 0 ? void 0 : _monacoEditor$getMode2[0];
    if (matchedResult) {
      setTimeout(function () {
        monacoEditor.revealLineInCenter(matchedResult.range.startLineNumber);
        monacoEditor.setPosition({
          column: matchedResult.range.endColumn,
          lineNumber: matchedResult.range.endLineNumber
        });
        monacoEditor.focus();
      }, 100);
    }
  };
  _proto.addFunction = function addFunction(params) {
    var _monacoEditor$getMode3;
    var monacoEditor = this.monacoEditor,
      monaco = this.monaco;
    if (!monacoEditor || !monaco) {
      return;
    }

    // 找到最后一个 }，在他前面插入新的 function 字符串
    var matches = (_monacoEditor$getMode3 = monacoEditor.getModel()) === null || _monacoEditor$getMode3 === void 0 ? void 0 : _monacoEditor$getMode3.findMatches('}');
    var range = {};
    if (matches && matches.length > 0) {
      var _matches;
      var _ref6 = ((_matches = matches[matches.length - 1]) === null || _matches === void 0 ? void 0 : _matches.range) || {},
        startLineNumber = _ref6.startLineNumber,
        startColumn = _ref6.startColumn;
      range = new monaco.Range(startLineNumber, startColumn, startLineNumber, startColumn);
    }
    var functionCode = params.template ? params.template : "\n\t" + params.functionName + "(){\n\t}\n";
    monacoEditor.executeEdits('log-source', [{
      identifier: 'event_id',
      range: range,
      text: functionCode,
      forceMoveMarkers: true
    }]);
    params.functionName && this.focusByFunctionName(params);
  };
  _proto.render = function render() {
    var _this$state = this.state,
      code = _this$state.code,
      hasError = _this$state.hasError,
      errorInfo = _this$state.errorInfo;
    return /*#__PURE__*/React.createElement("div", {
      className: "plugin-code-editor-js plugin-code-editor-inner"
    }, /*#__PURE__*/React.createElement(MonacoEditor, {
      value: code,
      language: "typescript",
      height: "100%",
      path: "file:///src/lowcode/index.tsx",
      supportFullScreen: true,
      onChange: this._updateCode,
      editorDidMount: this.editorDidMount
    }), hasError && typeof errorInfo === 'string' ? /*#__PURE__*/React.createElement(ErrorTip, {
      errorInfo: errorInfo
    }) : null);
  };
  return JsEditor;
}(PureComponent);
